==> cronevents-0.0.39a6-py3-none-any.whl: cronevents/db/logs/postgres.py <==
import time
import queue
import datetime
import threading

from cronevents.db.logs.base import LoggerBase
from cronevents.db.postgres import get_postgres_from_env, Postgres


class PostgresLogger(LoggerBase):
    table_name: str = 'cronevents_log'

    def __init__(self, trigger_id):
        self.trigger_id = trigger_id
        self.db = get_postgres_from_env()

        self.queue = queue.Queue()
        self.current_index = -1
        self.last_log = time.time()

        self.thread = None

        self.__pinging = False
        self.pinger_thread = None

    def start_logger(self):
        self.thread = threading.Thread(target=self.logger)
        self.thread.start()

        self.pinger_thread = threading.Thread(target=self.pinger, daemon=True)
        self.pinger_thread.start()

    def stop_logger(self):
        self.__pinging = False
        self.queue.put(None)

        if self.thread:
            self.thread.join()
            self.thread = None

        if self.pinger_thread:
            self.pinger_thread.join()
            self.pinger_thread = None

        self.ping()

    def __del__(self):
        self.stop_logger()

    def __enter__(self):
        self.start_logger()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop_logger()
        if exc_type:
            print('exc_type', exc_type)
            print('exc_val', exc_val)
            print('exc_tb', exc_tb)

    def create_row(self, logs: list[str] | list[tuple[str, float]]):
        for log in logs:
            self.current_index += 1

            if isinstance(log, tuple):
                log, t = log
            else:
                t = time.time()

            yield {
                'trigger_id': self.trigger_id,
                'index': self.current_index,
                'line': log,
                # 'epoch': t,
                'utc_time': datetime.datetime.fromtimestamp(time.time(), tz=datetime.timezone.utc),
                # datetime.datetime.fromtimestamp(t, datetime.UTC)
            }

    def upload(self, logs: list[str] | list[tuple[str, float]]):
        table = list(self.create_row(logs))
        self.upload_to_db(table)

    def upload_to_db(self, table):
        try:
            upload_logs(self.db, self.trigger_id, table)
        except:
            self.db = get_postgres_from_env()
            upload_logs(self.db, self.trigger_id, table)

    def log(self, s):
        self.queue.put(s)

    def logger(self):
        current_log = []
        while True:
            log = self.queue.get()
            if log is None:
                break

            current_log.append((log, time.time()))

            if time.time() - self.last_log > 1 or len(current_log) > 100:
                self.upload(current_log)
                current_log = []
                self.last_log = time.time()

        if current_log:
            self.upload(current_log)

    def ping(self):
        row = {
            'trigger_id': self.trigger_id,
            'index': -1,
            'line': 'ping',
            'utc_time': datetime.datetime.fromtimestamp(time.time(), tz=datetime.timezone.utc),
        }
        self.upload_to_db([row])

    __pinging = False

    def pinger(self):
        self.__pinging = True

        while self.__pinging:
            self.ping()
            time.sleep(60 * 5)


def upload_logs(db, trigger_id, logs):
    if logs:
        kwargs = {}
        index_query = f'create index if not exists event_logs_event_id_idx on {PostgresLogger.table_name} (trigger_id);'
        if isinstance(db, Postgres):
            index_query = f'create index if not exists event_logs_event_id_idx on {PostgresLogger.table_name} using hash (trigger_id);'
            kwargs['partition'] = 'trigger_id'
            kwargs['partition_query'] = f'''CREATE TABLE if not exists "{PostgresLogger.table_name}_{trigger_id}" 
                                    PARTITION OF "{PostgresLogger.table_name}" FOR VALUES IN ('{trigger_id}');'''

        db.upload_table(
            PostgresLogger.table_name,
            logs,
            id_column=['trigger_id', 'index'],
            **kwargs
        )
        db.query(index_query)

==> cronevents-0.0.40a9-py3-none-any.whl: cronevents/db/logs/postgres.py <==
import time
import queue
import datetime
import threading

from cronevents.db.logs.base import LoggerBase, Log, Generator
from cronevents.db.postgres import get_postgres_from_env, Postgres


class PostgresLogger(LoggerBase):
    table_name: str = 'cronevents_log'

    def __init__(self, trigger_id):
        self.trigger_id = trigger_id
        self.db = get_postgres_from_env()

        self.queue = queue.Queue()
        self.current_index = -1
        self.last_log = time.time()

        self.thread = None

        self.__pinging = False
        self.pinger_thread = None

    def start_logger(self):
        self.thread = threading.Thread(target=self.logger)
        self.thread.start()

        self.pinger_thread = threading.Thread(target=self.pinger, daemon=True)
        self.pinger_thread.start()

    def stop_logger(self):
        self.__pinging = False
        self.queue.put(None)

        if self.thread:
            self.thread.join()
            self.thread = None

        if self.pinger_thread:
            self.pinger_thread.join()
            self.pinger_thread = None

        self.ping()

    def __del__(self):
        self.stop_logger()

    def __enter__(self):
        self.start_logger()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop_logger()
        if exc_type:
            print('exc_type', exc_type)
            print('exc_val', exc_val)
            print('exc_tb', exc_tb)

    def create_row(self, logs: list[str] | list[tuple[str, float]]):
        for log in logs:
            self.current_index += 1

            if isinstance(log, tuple):
                log, t = log
            else:
                t = time.time()

            # yield {
            #     'trigger_id': self.trigger_id,
            #     'index': self.current_index,
            #     'line': log,
            #     # 'epoch': t,
            #     'utc_time': datetime.datetime.fromtimestamp(time.time(), tz=datetime.timezone.utc),
            #     # datetime.datetime.fromtimestamp(t, datetime.UTC)
            # }

            yield Log(
                trigger_id=self.trigger_id,
                index=self.current_index,
                line=log,
                utc_time=datetime.datetime.fromtimestamp(time.time(), tz=datetime.timezone.utc)
            ).to_row(datetime_ok=True)

    def upload(self, logs: list[str] | list[tuple[str, float]]):
        table = list(self.create_row(logs))
        self.upload_to_db(table)

    def upload_to_db(self, table):
        try:
            upload_logs(self.db, self.trigger_id, table)
        except:
            self.db = get_postgres_from_env()
            upload_logs(self.db, self.trigger_id, table)

    def log(self, s):
        self.queue.put(s)

    def logger(self):
        current_log = []
        while True:
            log = self.queue.get()
            if log is None:
                break

            current_log.append((log, time.time()))

            if time.time() - self.last_log > 1 or len(current_log) > 100:
                self.upload(current_log)
                current_log = []
                self.last_log = time.time()

        if current_log:
            self.upload(current_log)

    def ping(self):
        row = Log(
            trigger_id=self.trigger_id,
            index=-1,
            line='ping',
            utc_time=datetime.datetime.fromtimestamp(time.time(), tz=datetime.timezone.utc),
        ).to_row(datetime_ok=True)
        self.upload_to_db([row])

    __pinging = False

    def pinger(self):
        self.__pinging = True

        while self.__pinging:
            self.ping()
            time.sleep(60 * 5)

    def list(self, stream: bool = False) -> list[Log] | Generator[Log, None, None]:
        q = f"select * from {self.table_name} where trigger_id = $id${self.trigger_id}$id$ order by index asc;"
        if stream:
            return (Log.from_row(row) for row in self.db.download_table(sql=q, stream=True))
        return [Log.from_row(row) for row in self.db.download_table(sql=q)]


def upload_logs(db, trigger_id, logs):
    if logs:
        kwargs = {}
        index_query = f'create index if not exists event_logs_event_id_idx on {PostgresLogger.table_name} (trigger_id);'
        if isinstance(db, Postgres):
            index_query = f'create index if not exists event_logs_event_id_idx on {PostgresLogger.table_name} using hash (trigger_id);'
            kwargs['partition'] = 'trigger_id'
            kwargs['partition_query'] = f'''CREATE TABLE if not exists "{PostgresLogger.table_name}_{trigger_id}" 
                                    PARTITION OF "{PostgresLogger.table_name}" FOR VALUES IN ('{trigger_id}');'''

        db.upload_table(
            PostgresLogger.table_name,
            logs,
            id_column=['trigger_id', 'index'],
            **kwargs
        )
        db.query(index_query)
